@@ -3,9 +3,9 @@
 use flume::{bounded, Receiver, Sender};
 use futures::future::BoxFuture;
 use futures::io;
-use parking_lot::Mutex;
-use pi_cache::{Cache, Data, Metrics};
+use pi_cache::{Cache, Data, Metrics, Iter};
 use pi_hash::XHashMap;
+use pi_share::{ShareWeak, Share, ShareMutex};
 use pi_time::now_millisecond;
 use core::fmt;
 use std::collections::hash_map::Entry;
@@ -14,7 +14,6 @@ use std::hash::Hash;
 use std::io::Result;
 use std::ops::Deref;
 use std::result::Result as Result1;
-use std::sync::{Arc, Weak};
 
 /// 资产定义
 pub trait Asset: 'static {
@@ -57,8 +56,8 @@ pub(crate) struct AssetTable<A: Asset> {
     pub timeout: usize,
 }
 pub(crate) enum AssetResult<A: Asset> {
-    Ok(Weak<Droper<A>>),
-    Wait(Vec<Sender<Result<ArcDrop<A>>>>),
+    Ok(ShareWeak<Droper<A>>),
+    Wait(Vec<Sender<Result<Handle<A>>>>),
 }
 
 impl<A: Asset> AssetTable<A> {
@@ -80,39 +79,39 @@ impl<A: Asset> AssetTable<A> {
         self.cache.metrics()
     }
     /// 获得缓存的数量
-    pub fn cache_count(&self) -> usize {
+    pub fn cache_len(&self) -> usize {
         self.cache.count()
     }
-    /// 获得正在使用的数量
-    pub fn use_count(&self) -> usize {
-        self.map.len()
-    }
     /// 判断是否有指定键的数据
     pub fn contains_key(&self, k: &A::Key) -> bool {
         self.map.contains_key(k) || self.cache.contains_key(k)
     }
+    /// 获得缓存的迭代器
+    pub fn cache_iter(&self) -> Iter<'_, <A as Asset>::Key, Item<A>> {
+        self.cache.iter()
+    }
     /// 缓存指定的资产
     pub fn cache(&mut self, k: A::Key, v: A) -> Option<A> {
         self.cache.put(k, Item(v, self.timeout as u64 + now_millisecond())).map(|v| v.0)
     }
     /// 放入资产， 并获取资产句柄， 返回None表示已有条目
-    pub fn insert(&mut self, k: A::Key, v: A, lock: usize) -> Option<ArcDrop<A>> {
+    pub fn insert(&mut self, k: A::Key, v: A, lock: usize) -> Option<Handle<A>> {
         match self.map.entry(k) {
             Entry::Occupied(_) => None,
             Entry::Vacant(e) => {
                 // self.size += v.size();
-                let r = Arc::new(Droper {
+                let r = Share::new(Droper {
                     key: e.key().clone(),
                     data: Some(v),
                     lock,
                 });
-                e.insert(AssetResult::Ok(Arc::downgrade(&r)));
+                e.insert(AssetResult::Ok(Share::downgrade(&r)));
                 Some(r)
             }
         }
     }
     /// 获取已经存在或被缓存的资产
-    pub fn get(&mut self, k: A::Key, lock: usize) -> Option<Option<ArcDrop<A>>> {
+    pub fn get(&mut self, k: A::Key, lock: usize) -> Option<Option<Handle<A>>> {
         match self.map.entry(k) {
             Entry::Occupied(e) => match e.get() {
                 AssetResult::Ok(r) => Some(r.upgrade()),
@@ -124,12 +123,12 @@ impl<A: Asset> AssetTable<A> {
                     None => return None,
                 };
                 // self.size += v.0.size();
-                let r = Arc::new(Droper {
+                let r = Share::new(Droper {
                     key: e.key().clone(),
                     data: Some(v.0),
                     lock,
                 });
-                e.insert(AssetResult::Ok(Arc::downgrade(&r)));
+                e.insert(AssetResult::Ok(Share::downgrade(&r)));
                 Some(Some(r))
             }
         }
@@ -140,7 +139,7 @@ impl<A: Asset> AssetTable<A> {
         k: A::Key,
         lock: usize,
         wait: bool,
-    ) -> Result1<Option<ArcDrop<A>>, Option<Receiver<Result<ArcDrop<A>>>>> {
+    ) -> Result1<Option<Handle<A>>, Option<Receiver<Result<Handle<A>>>>> {
         match self.map.entry(k) {
             Entry::Occupied(mut e) => match e.get_mut() {
                 AssetResult::Ok(r) => Ok(r.upgrade()),
@@ -153,12 +152,12 @@ impl<A: Asset> AssetTable<A> {
             Entry::Vacant(e) => match self.cache.take(e.key()) {
                 Some(v) => {
                     // self.size += v.0.size();
-                    let r = Arc::new(Droper {
+                    let r = Share::new(Droper {
                         key: e.key().clone(),
                         data: Some(v.0),
                         lock,
                     });
-                    e.insert(AssetResult::Ok(Arc::downgrade(&r)));
+                    e.insert(AssetResult::Ok(Share::downgrade(&r)));
                     Ok(Some(r))
                 }
                 None => {
@@ -176,38 +175,42 @@ impl<A: Asset> AssetTable<A> {
         k: A::Key,
         v: A,
         lock: usize,
-    ) -> (ArcDrop<A>, Option<AssetResult<A>>) {
+    ) -> (Handle<A>, Option<AssetResult<A>>) {
         // self.size += v.size();
-        let r = Arc::new(Droper {
+        let r = Share::new(Droper {
             key: k.clone(),
             data: Some(v),
             lock,
         });
-        let weak = AssetResult::Ok(Arc::downgrade(&r));
+        let weak = AssetResult::Ok(Share::downgrade(&r));
         (r, self.map.insert(k, weak))
     }
     /// 超时整理方法， 清理最小容量外的超时资产
-    pub fn timeout_collect<G: Garbageer<A>>(&mut self, g: &G, capacity: usize, now: u64) -> usize {
+    pub fn timeout_collect<G: Garbageer<A>>(&mut self, g: &G, capacity: usize, now: u64) -> (usize, usize) {
+        let mut l = 0;
         let mut s = 0;
         for r in self.cache.timeout_collect(capacity, now) {
+            l += 1;
             s += r.1 .0.size();
             g.garbage(r.0, r.1 .0)
         }
-        s
+        (l, s)
     }
     /// 超量整理方法， 按照先进先出的原则，清理超出容量的资产
-    pub fn capacity_collect<G: Garbageer<A>>(&mut self, g: &G, capacity: usize) -> usize {
+    pub fn capacity_collect<G: Garbageer<A>>(&mut self, g: &G, capacity: usize) -> (usize, usize) {
+        let mut l = 0;
         let mut s = 0;
         for r in self.cache.capacity_collect(capacity) {
+            l += 1;
             s += r.1 .0.size();
             g.garbage(r.0, r.1 .0)
         }
-        s
+        (l, s)
     }
 }
 
 /// 资产条目
-struct Item<A: Asset>(A, u64);
+pub(crate) struct Item<A: Asset>(pub A, pub u64);
 impl<A: Asset> Data for Item<A> {
     fn size(&self) -> usize {
         self.0.size()
@@ -218,7 +221,7 @@ impl<A: Asset> Data for Item<A> {
 }
 
 /// 可拷贝可回收的资产句柄
-pub type ArcDrop<T> = Arc<Droper<T>>;
+pub type Handle<T> = Share<Droper<T>>;
 
 pub struct Droper<T: Asset> {
     key: T::Key,
@@ -243,7 +246,7 @@ impl<T: Asset> Deref for Droper<T> {
 impl<T: Asset> Drop for Droper<T> {
     fn drop(&mut self) {
         let v = unsafe { self.data.take().unwrap_unchecked() };
-        let lock: &Mutex<AssetTable<T>> = unsafe { &*(self.lock as *const Mutex<AssetTable<T>>) };
+        let lock: &ShareMutex<AssetTable<T>> = unsafe { &*(self.lock as *const ShareMutex<AssetTable<T>>) };
         let mut table = lock.lock();
         // table.size -= v.size();
         table.map.remove(&self.key);
